@@ -2,7 +2,7 @@ use strict;
 use warnings;
 
 package XML::Sig;
-our $VERSION = '0.64';
+our $VERSION = '0.65';
 
 use Encode;
 # ABSTRACT: XML::Sig - A toolkit to help sign and verify XML Digital Signatures
@@ -181,21 +181,25 @@ sub sign {
         my $reference = $signid; #$self->{parser}->findvalue('//@ID', $xml);
         print ("   Reference URI: $reference\n") if $DEBUG;
 
-        # Add the Signature to the xml being signed
-        $xml->appendWellBalancedChunk($signature_xml, 'UTF-8');
-        $xml->setNamespace("http://www.w3.org/2000/09/xmldsig#", "dsig", 0);
+        local $XML::LibXML::skipXMLDeclaration = $self->{ no_xml_declaration };
+
+        my $signature_dom = XML::LibXML->load_xml( string => $signature_xml );
+
+        my $xpath = XML::LibXML::XPathContext->new($signature_dom);
+        $xpath->registerNs('dsig', 'http://www.w3.org/2000/09/xmldsig#');
+        $xpath->registerNs('ec', 'http://www.w3.org/2001/10/xml-exc-c14n#');
 
         # Canonicalize the SignedInfo to http://www.w3.org/TR/2001/REC-xml-c14n-20010315#WithComments
         # TODO Change the Canonicalization method in the xml fragment from _signedinfo_xml
 
-        my ($signature_node) = $xml->findnodes(
-            './dsig:Signature', $xml);
-        my ($signed_info_node) = $xml->findnodes(
-            './dsig:Signature/dsig:SignedInfo',$xml);
+        my ($signature_node) = $xpath->findnodes(
+            '/dsig:Signature', $signature_xml);
+        my ($signed_info_node) = $xpath->findnodes(
+            '/dsig:Signature/dsig:SignedInfo',$signature_xml);
 
         # Add the digest value to the Signed info
-        my ($digest_value_node) = $xml->findnodes(
-            './dsig:Signature/dsig:SignedInfo/dsig:Reference/dsig:DigestValue', $signature_node);
+        my ($digest_value_node) = $xpath->findnodes(
+            '/dsig:Signature/dsig:SignedInfo/dsig:Reference/dsig:DigestValue', $signature_xml);
         $digest_value_node->removeChildNodes();
         $digest_value_node->appendText($digest);
 
@@ -220,11 +224,20 @@ sub sign {
         }
 
         # Add the Signature to the SignatureValue
-        my ($signature_value_node) = $xml->findnodes(
-            './dsig:Signature/dsig:SignatureValue', $signature_node);
+        my ($signature_value_node) = $xpath->findnodes(
+            '/dsig:Signature/dsig:SignatureValue', $signature_xml);
         $signature_value_node->removeChildNodes();
         $signature_value_node->appendText($signature);
 
+        my $set = $xpath->findnodes('dsig:Signature');
+
+        my $node = $set->get_node(1)->cloneNode( 1 );
+
+        my $root = $dom->findnodes("//*[\@ID=\'$signid\']");
+
+        my $loc = $root->shift();
+        $loc->addChild($node);
+
         print ("\n\n\n SignatureValue:\n" . $signature_value_node . "\n\n\n") if $DEBUG;
     }
 
@@ -1018,35 +1031,6 @@ sub _get_node {
     }
 }
 
-# TODO remove unused?
-sub _get_node_as_text {
-    my $self = shift;
-    my ($xpath, $context) = @_;
-    my $node = $self->_get_node($xpath, $context);
-    if ($node) {
-        return $node->toString;
-    } else {
-        return '';
-    }
-}
-
-# TODO remove unused?
-sub _transform_env_sig {
-    my $self = shift;
-    my ($str) = @_;
-    my $prefix = '';
-    if (defined $self->{dsig_prefix} && length $self->{dsig_prefix}) {
-        $prefix = $self->{dsig_prefix} . ':';
-    }
-
-    # This removes the first Signature tag from the XML - even if there is another XML tree with another Signature inside and that comes first.
-    # TODO: Remove the outermost Signature only.
-
-    $str =~ s/(<${prefix}Signature(.*?)>(.*?)\<\/${prefix}Signature>)//is;
-
-    return $str;
-}
-
 ##
 ## _trim($string)
 ##
@@ -1701,7 +1685,7 @@ XML::Sig - XML::Sig - A toolkit to help sign and verify XML Digital Signatures
 
 =head1 VERSION
 
-version 0.64
+version 0.65
 
 =head1 SYNOPSIS
 
